VAR_NAVER_STATEMENT = 'var naver='
HEADERS = {
    "accept": "*/*",
    "accept-encoding": "gzip, deflate, br",
    "accept-language": "ko",
    "Content-Type": "application/json",
    "referer": "https://pcmap.place.naver.com/",
    "Origin": "https://pcmap.place.naver.com",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36"
}




def parse_naver_var_in_script_texts(script_texts):
    naver_string = None
    for script_text in script_texts:
        if script_text.string is None:
            continue
        if VAR_NAVER_STATEMENT in script_text.string:
            naver_string = script_text.string
            break

    return naver_string




def create(id:str, page_no:int, page_cnt:int):
    operation_name = "getVisitorReviews"
    variables = _define_variables(id, page_no, page_cnt)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(id:str, page_no: int, page_cnt:int):
    return {
        "input": {
            "businessId": id,
            "bookingBusinessId": "null",
            "display": page_cnt,
            "page": page_no,
            "includeContent": True,
        },
        "id": id,
    }

QUERY_STATEMENT = """query getVisitorReviews($input: VisitorReviewsInput) {
                                visitorReviews(input: $input) {
                                    items {
                                    id
                                    rating
                                    author {
                                        id
                                        nickname
                                        from
                                        imageUrl
                                        objectId
                                        url
                                        review {
                                        totalCount
                                        imageCount
                                        avgRating
                                        __typename
                                        }
                                        theme {
                                        totalCount
                                        __typename
                                        }
                                        __typename
                                    }
                                    body
                                    thumbnail
                                    media {
                                        type
                                        thumbnail
                                        class
                                        __typename
                                    }
                                    tags
                                    status
                                    visitCount
                                    viewCount
                                    visited
                                    created
                                    reply {
                                        editUrl
                                        body
                                        editedBy
                                        created
                                        replyTitle
                                        __typename
                                    }
                                    originType
                                    item {
                                        name
                                        code
                                        options
                                        __typename
                                    }
                                    language
                                    highlightOffsets
                                    apolloCacheId
                                    translatedText
                                    businessName
                                    showBookingItemName
                                    showBookingItemOptions
                                    bookingItemName
                                    bookingItemOptions
                                    votedKeywords {
                                        code
                                        iconUrl
                                        iconCode
                                        displayName
                                        __typename
                                    }
                                    userIdno
                                    isFollowing
                                    followerCount
                                    followRequested
                                    loginIdno
                                    receiptInfoUrl
                                    __typename
                                    }
                                    starDistribution {
                                    score
                                    count
                                    __typename
                                    }
                                    hideProductSelectBox
                                    total
                                    showRecommendationSort
                                    __typename
                                }
                                }"""


import json
import re
import requests
from . import query
from bs4 import BeautifulSoup

from .utils import HEADERS


class AnnouncementMixin:

    def get_announcement(self, business_id:str, business_type:str='restaurant', device_type:str='pcmap', proxy=None):
        data = query.get_announcements.create(business_id, business_type, device_type)

        response = requests.post("https://pcmap-api.place.naver.com/graphql",
                                 headers=HEADERS,
                                 data=json.dumps(data),
                                 proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']
        graphql_data['business_id'] = business_id

        return graphql_data



import json
import re
import requests
from . import query
from bs4 import BeautifulSoup

from .utils import HEADERS, parse_naver_var_in_script_texts


class PlaceMixin:

    def search_places(self, keyword: str, page_no: int, page_size: int, proxy=None):
        data = query.get_restaurants.create(keyword, page_no, page_size)
        print("proxy: ", proxy)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['restaurants']
        return graphql_data

    def get_place(self, business_id, proxy=None):
        # url = f"https://pcmap.place.naver.com/restaurant/{business_id}/home"
        url = "https://pcmap.place.naver.com/restaurant/{}".format(business_id)
        response = requests.get(url, proxies={"http":proxy})
        response.raise_for_status()
        html_text = response.content

        broadcasts = self.__parse_broadcast_in_html(html_text)
        return broadcasts

    def get_restaurant(self, business_id, proxy):
        data = query.get_restaurant.create(business_id)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']
        return graphql_data

    def __parse_broadcast_in_html(self, html_text: str):
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")

        naver_var = parse_naver_var_in_script_texts(scripts)
        variable_name = 'window.__APOLLO_STATE__'
        pattern = re.compile(rf'\b{re.escape(variable_name)}\s*=\s*(.*?)(\n)')
        match = pattern.search(naver_var)

        if match:
            data = match.group(1)
            return json.loads(data[:-1])
        else:
            return None


    def get_place_summary(self, id: str, proxy=None):
        url = f"https://map.naver.com/v5/api/sites/summary/" + id + "?lang=ko"
        data = {
            "language": 'kor',
            "order_by": "2"
        }
        response = requests.get(url, data=data, proxies={"http": proxy})
        response.raise_for_status()
        resposne_data = response.json()
        resposne_data['_id'] = id
        return resposne_data

    def get_menus(self, business_id, proxy=None):
        url = f"https://pcmap.place.naver.com/restaurant/{business_id}/menu/list"
        response = requests.get(url, proxies={"http":proxy})
        response.raise_for_status()
        html_text = response.content
        menus = self.__parse_menus_internal(html_text)
        result = {
            "business_id": business_id,
            "menus": menus
        }
        return result

    def __parse_menus_internal(self, html_text: str):
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")
        naver_string = None

        for s in scripts:
            if s.string is None:
                continue
            if "var naver=" in s.string:
                naver_string = s.string

        menu_regex = re.compile(r'\{"__typename":"Menu".*?\}')
        menu_string_list = re.findall(menu_regex, naver_string)

        # 일반
        menus = []
        for menu_string in menu_string_list:
            menu_json = json.loads(menu_string)
            menu = {
                "name": menu_json["name"],
                "price": menu_json["price"],
                "description": menu_json["description"],
                "imageUrl": menu_json["images"][0] if menu_json["images"] else "",
                "isRecommended": menu_json["recommend"],
            }
            menus.append(menu)
        # baemin
        baemin_menu_regex = re.compile(r'\{"__typename":"BaeminMenu".*?\}')
        baemin_menu_string_list = re.findall(baemin_menu_regex, naver_string)

        for baemin_menu_string in baemin_menu_string_list:
            baemin_menu_json = json.loads(baemin_menu_string)
            menu = {
                "name":baemin_menu_json["name"],
                "price":baemin_menu_json["price"],
                "imageUrl":menu_json["images"][0] if menu_json["images"] else "",

            }
            menus.append(menu)
        return menus



def create(id: str, page_no:int, page_size:int):
    query = {
        "operationName": "getVisitorReviewThemeLists",
        "variables": _define_variables(id, page_no, page_size),
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(id: str, page_no:int, page_size:int):
    return {
        "input":{
            "businessId": id,
            "display":page_size,
            "page": page_no
        }
    }


QUERY_STATEMENT = """
query getVisitorReviewThemeLists($input: ThemeListsInput) {
  themeLists(input: $input) {
    themeLists {
      id
      title
      viewCount
      itemCount
      reviews {
        businessName
        reviewBody
        imageUrl
        __typename
      }
      authorNickname
      authorImageUrl
      isFollowing
      themeListUrl
      authorUrl
      __typename
    }
    total
    __typename
  }
}

"""



import re
import json
import requests
from bs4 import BeautifulSoup
from . import query

from .utils import HEADERS, parse_naver_var_in_script_texts


class BookingMixin:
    def get_booking_items(self, business_id, business_type='restaurant', biz_type='STANDARD', proxy=None):
        data = query.get_booking_items.create(business_id, business_type, biz_type)
        response = requests.post("https://pcmap-api.place.naver.com/graphql",
                                 headers=HEADERS,
                                 data=json.dumps(data),
                                 proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']
        graphql_data['business_id'] = business_id
        return graphql_data



from . import get_visitor_reviews
from . import get_visitor_review_stats
from . import get_ugc_reviews
from . import get_restaurants
from . import get_visitor_review_theme_lists
from . import get_visitor_review_photos_in_visitor_review_tab
from . import get_promotions
from . import get_booking_items
from . import get_announcements
from . import get_popular_menus
from . import get_restaurant


Query = {
    "get_announcement": get_announcements,
    "get_restaurants": get_restaurants,
    "get_popular_menus": get_popular_menus,
    "get_visitor_reviews": get_visitor_reviews,
    "get_ugc_reviews": get_ugc_reviews,
    "get_visitor_review_stats": get_visitor_review_stats,
    "get_visitor_review_photos_in_visitor_review_tab": get_visitor_review_photos_in_visitor_review_tab,
    "get_visitor_review_theme_lists": get_visitor_review_theme_lists,
    "get_promotions": get_promotions,
    "get_booking_items":get_booking_items,
    "get_restaurant": get_restaurant
}

__all__ = [get_restaurants]


import logging

from .mixin.announce import AnnouncementMixin
from .mixin.booking import BookingMixin
from .mixin.menu import MenuMixin
from .mixin.place import PlaceMixin
from .mixin.promotions import PromotionsMixin
from .mixin.review import ReviewMixin
from .mixin.broadcast import BroadcastMixin

__VERSION__ = "0.1.8"

DEFAULT_LOGGER = logging.getLogger("naverplaceapi")

class Client(
    PlaceMixin,
    ReviewMixin,
    BroadcastMixin,
    PromotionsMixin,
    AnnouncementMixin,
    BookingMixin,
    MenuMixin
):
    pass



import json

import requests

from . import query
from naverplaceapi.mixin.utils import HEADERS


class ReviewMixin:
    def get_visitor_reviews(self, business_id: str, page_no: int, page_cnt: int, proxy=None):
        data = query.get_visitor_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data  ={}
        # ['visitorReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_ugc_reviews(self, business_id: str, page_no: int, page_cnt: int, proxy=None):
        data = query.get_ugc_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['restaurant']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_stats(self, business_id: str, proxy=None):
        data = query.get_visitor_review_stats.create(business_id)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviewStats']
        if graphql_data is None:
            return None
        graphql_data['_id'] = graphql_data['id']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_photos_in_visitor_review_tab(self, store_id: str, page_no: int, page_size: int, proxy=None):
        data = query.get_visitor_review_photos_in_visitor_review_tab.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data = {}
        graphql_data['business_id'] = store_id
        return graphql_data

    def get_visitor_review_theme_lists(self, store_id: str, page_no, page_size, proxy=None):
        data = query.get_visitor_review_theme_lists.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies={"http":proxy})
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['themeLists']
        graphql_data['business_id'] = store_id

        return graphql_data


